﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using ELTE.TodoList.Persistence.Services;
using ELTE.TodoList.DTO;
using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using ELTE.TodoList.Persistence;

namespace ELTE.TodoList.WebApi.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ItemsController : ControllerBase
    {
        private readonly ITodoListService _service;
        private readonly IMapper _mapper;

        public ItemsController(ITodoListService service, IMapper mapper)
        {
            _service = service;
            _mapper = mapper;
        }

        // GET: api/Items
        [HttpGet]
        public ActionResult<IEnumerable<ItemDto>> GetItems(int listId)
        {
            try
            {
                return _service
                    .GetListById(listId)
                    .Items
                    .Select(item => _mapper.Map<ItemDto>(item))
                    .ToList();
            }
            catch (Exception)
            {
                return NotFound();
            }
        }

        [HttpGet("{id}")]
        public ActionResult<ItemDto> GetItem(Int32 id)
        {
            try
            {
                var item = _service.GetItem(id);
                return _mapper.Map<ItemDto>(item);
            }
            catch (InvalidOperationException)
            {
                return NotFound();
            }
        }

        // PUT: api/Items/5
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for
        // more details see https://aka.ms/RazorPagesCRUD.
        [Authorize]
        [HttpPut("{id}")]
        public IActionResult PutItem(Int32 id, ItemDto itemDto)
        {
            if (id != itemDto.Id)
            {
                return BadRequest();
            }

            var item = _mapper.Map<Item>(itemDto);

            if (_service.UpdateItem(item))
            {
                return Ok();
            }
            else
            {
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }

        // POST: api/Items
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for
        // more details see https://aka.ms/RazorPagesCRUD.
        [Authorize]
        [HttpPost]
        public ActionResult<ListDto> PostItem(ItemDto itemDto)
        {
            var item = _mapper.Map<Item>(itemDto);
            var newItem = _service.CreateItem(item);

            if (newItem is null)
            {
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
            else
            {
                return CreatedAtAction(nameof(GetItem), new { id = item.Id },
                    _mapper.Map<ItemDto>(item));
            }
        }

        [Authorize]
        [HttpDelete("{id}")]
        public IActionResult DeleteItem(int id)
        {
            if (_service.DeleteItem(id))
            {
                return Ok();
            }
            else
            {
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }
    }
}
